@@ -555,6 +555,8 @@ gcm_apply_create_icc_profile_for_edid (GsdColorManager *manager,
         cmsToneCurve *transfer_curve[3] = { NULL, NULL, NULL };
         const gchar *data;
         gboolean ret = FALSE;
+        gchar *str;
+        gfloat edid_gamma;
         gfloat localgamma;
 #ifdef HAVE_NEW_LCMS
         cmsHANDLE dict = NULL;
@@ -691,6 +693,32 @@ gcm_apply_create_icc_profile_for_edid (GsdColorManager *manager,
         data = gcm_edid_get_vendor_name (edid);
         if (data != NULL)
                 _cmsDictAddEntryAscii (dict, "EDID_manufacturer", data);
+        edid_gamma = gcm_edid_get_gamma (edid);
+        if (edid_gamma > 0.0 && edid_gamma < 10.0) {
+                str = g_strdup_printf ("%f", edid_gamma);
+                _cmsDictAddEntryAscii (dict, "EDID_gamma", str);
+                g_free (str);
+        }
+
+        /* also add the primaries */
+        str = g_strdup_printf ("%f", chroma.Red.x);
+        _cmsDictAddEntryAscii (dict, "EDID_red_x", str);
+        g_free (str);
+        str = g_strdup_printf ("%f", chroma.Red.y);
+        _cmsDictAddEntryAscii (dict, "EDID_red_y", str);
+        g_free (str);
+        str = g_strdup_printf ("%f", chroma.Green.x);
+        _cmsDictAddEntryAscii (dict, "EDID_green_x", str);
+        g_free (str);
+        str = g_strdup_printf ("%f", chroma.Green.y);
+        _cmsDictAddEntryAscii (dict, "EDID_green_y", str);
+        g_free (str);
+        str = g_strdup_printf ("%f", chroma.Blue.x);
+        _cmsDictAddEntryAscii (dict, "EDID_blue_x", str);
+        g_free (str);
+        str = g_strdup_printf ("%f", chroma.Blue.y);
+        _cmsDictAddEntryAscii (dict, "EDID_blue_y", str);
+        g_free (str);
 
         /* write new tag */
         ret = cmsWriteTag (lcms_profile, cmsSigMetaTag, dict);
@@ -1197,35 +1225,33 @@ gcm_session_device_assign_connect_cb (GObject *object,
                 goto out;
         }
 
-        /* get the output EDID */
+        /* create profile from device edid if it exists */
         edid = gcm_session_get_output_edid (manager, output, &error);
         if (edid == NULL) {
                 g_warning ("unable to get EDID for %s: %s",
                            cd_device_get_id (device),
                            error->message);
-                g_error_free (error);
-                goto out;
-        }
+                g_clear_error (&error);
 
-        /* create profile from device edid if it does not exist */
-        autogen_filename = g_strdup_printf ("edid-%s.icc",
-                                            gcm_edid_get_checksum (edid));
-        autogen_path = g_build_filename (g_get_user_data_dir (),
-                                         "icc", autogen_filename, NULL);
-
-        if (g_file_test (autogen_path, G_FILE_TEST_EXISTS)) {
-                g_debug ("auto-profile edid %s exists", autogen_path);
         } else {
-                g_debug ("auto-profile edid does not exist, creating as %s",
-                         autogen_path);
-                ret = gcm_apply_create_icc_profile_for_edid (manager,
-                                                             edid,
-                                                             autogen_path,
-                                                             &error);
-                if (!ret) {
-                        g_warning ("failed to create profile from EDID data: %s",
-                                     error->message);
-                        g_clear_error (&error);
+                autogen_filename = g_strdup_printf ("edid-%s.icc",
+                                                    gcm_edid_get_checksum (edid));
+                autogen_path = g_build_filename (g_get_user_data_dir (),
+                                                 "icc", autogen_filename, NULL);
+                if (g_file_test (autogen_path, G_FILE_TEST_EXISTS)) {
+                        g_debug ("auto-profile edid %s exists", autogen_path);
+                } else {
+                        g_debug ("auto-profile edid does not exist, creating as %s",
+                                 autogen_path);
+                        ret = gcm_apply_create_icc_profile_for_edid (manager,
+                                                                     edid,
+                                                                     autogen_path,
+                                                                     &error);
+                        if (!ret) {
+                                g_warning ("failed to create profile from EDID data: %s",
+                                             error->message);
+                                g_clear_error (&error);
+                        }
                 }
         }
 
@@ -1339,9 +1365,9 @@ gcm_session_create_device_cb (GObject *object,
 static void
 gcm_session_add_x11_output (GsdColorManager *manager, GnomeRROutput *output)
 {
-        const gchar *model;
-        const gchar *serial;
-        const gchar *vendor;
+        const gchar *model = NULL;
+        const gchar *serial = NULL;
+        const gchar *vendor = NULL;
         gboolean ret;
         gchar *device_id = NULL;
         GcmEdid *edid;
@@ -1349,37 +1375,38 @@ gcm_session_add_x11_output (GsdColorManager *manager, GnomeRROutput *output)
         GHashTable *device_props = NULL;
         GsdColorManagerPrivate *priv = manager->priv;
 
-        /* get edid */
+        /* try to get edid */
         edid = gcm_session_get_output_edid (manager, output, &error);
         if (edid == NULL) {
                 g_warning ("failed to get edid: %s",
                            error->message);
-                g_error_free (error);
-                goto out;
+                g_clear_error (&error);
         }
 
-        /* is this an internal device? */
+        /* prefer DMI data for the internal output */
         ret = gnome_rr_output_is_laptop (output);
         if (ret) {
                 model = gcm_dmi_get_name (priv->dmi);
                 vendor = gcm_dmi_get_vendor (priv->dmi);
-        } else {
-                model = gcm_edid_get_monitor_name (edid);
-                if (model == NULL)
-                        model = gnome_rr_output_get_name (output);
-                vendor = gcm_edid_get_vendor_name (edid);
         }
 
-        /* get a serial number if one exists */
-        serial = gcm_edid_get_serial_number (edid);
-        if (serial == NULL)
-                serial = "unknown";
+        /* use EDID data if we have it */
+        if (edid != NULL) {
+                if (model == NULL)
+                        model = gcm_edid_get_monitor_name (edid);
+                if (vendor == NULL)
+                        vendor = gcm_edid_get_vendor_name (edid);
+                if (serial == NULL)
+                        serial = gcm_edid_get_serial_number (edid);
+        }
 
         /* ensure mandatory fields are set */
         if (model == NULL)
-                model = "unknown";
+                model = gnome_rr_output_get_name (output);
         if (vendor == NULL)
                 vendor = "unknown";
+        if (serial == NULL)
+                serial = "unknown";
 
         device_id = gcm_session_get_output_id (manager, output);
         g_debug ("output %s added", device_id);
@@ -1413,7 +1440,6 @@ gcm_session_add_x11_output (GsdColorManager *manager, GnomeRROutput *output)
                                  NULL,
                                  gcm_session_create_device_cb,
                                  manager);
-out:
         g_free (device_id);
         if (device_props != NULL)
                 g_hash_table_unref (device_props);
@@ -2054,14 +2080,12 @@ gcm_session_profile_store_added_cb (GcmProfileStore *profile_store,
         /* generate ID */
         checksum = gcm_session_get_md5_for_filename (filename, &error);
         if (checksum == NULL) {
-                g_warning ("failed to get profile checksum: %s",
-                           error->message);
+                g_debug ("failed to get profile checksum for %s: %s",
+                         filename, error->message);
                 g_error_free (error);
                 goto out;
         }
-        profile_id = g_strdup_printf ("icc-%s-%s",
-                                      checksum,
-                                      g_get_user_name ());
+        profile_id = g_strdup_printf ("icc-%s", checksum);
         profile_props = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                NULL, NULL);
         g_hash_table_insert (profile_props,
