@@ -49,7 +49,7 @@
 #include "gsd-media-keys-manager.h"
 
 #include "shortcuts-list.h"
-#include "gsd-media-keys-window.h"
+#include "gsd-osd-window.h"
 #include "gsd-input-helper.h"
 #include "gsd-enums.h"
 
@@ -100,6 +100,10 @@ static const gchar introspection_xml[] =
 #define VOLUME_STEP 6           /* percents for one volume button press */
 #define MAX_VOLUME 65536.0
 
+#define GNOME_DESKTOP_INPUT_SOURCES_DIR "org.gnome.desktop.input-sources"
+#define KEY_CURRENT_INPUT_SOURCE "current"
+#define KEY_INPUT_SOURCES        "sources"
+
 #define GSD_MEDIA_KEYS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_MEDIA_KEYS_MANAGER, GsdMediaKeysManagerPrivate))
 
 typedef struct {
@@ -342,7 +346,7 @@ dialog_init (GsdMediaKeysManager *manager)
         }
 
         if (manager->priv->dialog == NULL) {
-                manager->priv->dialog = gsd_media_keys_window_new ();
+                manager->priv->dialog = gsd_osd_window_new ();
         }
 }
 
@@ -671,7 +675,6 @@ dialog_show (GsdMediaKeysManager *manager)
         int            screen_h;
         int            x;
         int            y;
-        GtkRequisition win_req;
         GdkRectangle   geometry;
         int            monitor;
 
@@ -684,12 +687,6 @@ dialog_show (GsdMediaKeysManager *manager)
          * know its true size, yet, so we need to jump through hoops
          */
         gtk_window_get_default_size (GTK_WINDOW (manager->priv->dialog), &orig_w, &orig_h);
-        gtk_widget_size_request (manager->priv->dialog, &win_req);
-
-        if (win_req.width > orig_w)
-                orig_w = win_req.width;
-        if (win_req.height > orig_h)
-                orig_h = win_req.height;
 
         monitor = gdk_screen_get_primary_monitor (manager->priv->current_screen);
 
@@ -840,7 +837,7 @@ do_eject_action (GsdMediaKeysManager *manager)
 
         /* Show the dialogue */
         dialog_init (manager);
-        gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
                                                  "media-eject-symbolic",
                                                  FALSE);
         dialog_show (manager);
@@ -902,7 +899,7 @@ static void
 do_touchpad_osd_action (GsdMediaKeysManager *manager, gboolean state)
 {
         dialog_init (manager);
-        gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
                                                  state ? "input-touchpad-symbolic" : "touchpad-disabled-symbolic",
                                                  FALSE);
         dialog_show (manager);
@@ -944,11 +941,11 @@ update_dialog (GsdMediaKeysManager *manager,
         }
 
         dialog_init (manager);
-        gsd_media_keys_window_set_volume_muted (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_volume_muted (GSD_OSD_WINDOW (manager->priv->dialog),
                                                 muted);
-        gsd_media_keys_window_set_volume_level (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog), vol);
-        gsd_media_keys_window_set_action (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
-                                          GSD_MEDIA_KEYS_WINDOW_ACTION_VOLUME);
+        gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog), vol);
+        gsd_osd_window_set_action (GSD_OSD_WINDOW (manager->priv->dialog),
+                                          GSD_OSD_WINDOW_ACTION_VOLUME);
         dialog_show (manager);
 
         if (quiet == FALSE && sound_changed != FALSE && muted == FALSE) {
@@ -1367,7 +1364,7 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
         if (!have_listeners) {
                 /* Popup a dialog with an (/) icon */
                 dialog_init (manager);
-                gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
                                                          "action-unavailable-symbolic",
                                                          FALSE);
                 dialog_show (manager);
@@ -1700,10 +1697,10 @@ update_screen_cb (GObject             *source_object,
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(u)", &percentage);
         dialog_init (manager);
-        gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
                                                  "display-brightness-symbolic",
                                                  TRUE);
-        gsd_media_keys_window_set_volume_level (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
                                                 percentage);
         dialog_show (manager);
         g_variant_unref (new_percentage);
@@ -1752,10 +1749,10 @@ update_keyboard_cb (GObject             *source_object,
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(u)", &percentage);
         dialog_init (manager);
-        gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
                                                  "keyboard-brightness-symbolic",
                                                  TRUE);
-        gsd_media_keys_window_set_volume_level (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+        gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
                                                 percentage);
         dialog_show (manager);
         g_variant_unref (new_percentage);
@@ -1782,6 +1779,40 @@ do_keyboard_brightness_action (GsdMediaKeysManager *manager,
                            manager);
 }
 
+static void
+do_switch_input_source_action (GsdMediaKeysManager *manager,
+                               MediaKeyType         type)
+{
+        GSettings *settings;
+        GVariant *sources;
+        gint i, n;
+
+        settings = g_settings_new (GNOME_DESKTOP_INPUT_SOURCES_DIR);
+        sources = g_settings_get_value (settings, KEY_INPUT_SOURCES);
+
+        n = g_variant_n_children (sources);
+        if (n < 2)
+                goto out;
+
+        i = g_settings_get_uint (settings, KEY_CURRENT_INPUT_SOURCE);
+
+        if (type == SWITCH_INPUT_SOURCE_KEY)
+                i += 1;
+        else
+                i -= 1;
+
+        if (i < 0)
+                i = n - 1;
+        else if (i >= n)
+                i = 0;
+
+        g_settings_set_uint (settings, KEY_CURRENT_INPUT_SOURCE, i);
+
+ out:
+        g_variant_unref (sources);
+        g_object_unref (settings);
+}
+
 static void
 do_custom_action (GsdMediaKeysManager *manager,
                   MediaKey            *key,
@@ -1945,6 +1976,10 @@ do_action (GsdMediaKeysManager *manager,
         case BATTERY_KEY:
                 do_execute_desktop (manager, "gnome-power-statistics.desktop", timestamp);
                 break;
+        case SWITCH_INPUT_SOURCE_KEY:
+        case SWITCH_INPUT_SOURCE_BACKWARD_KEY:
+                do_switch_input_source_action (manager, type);
+                break;
         /* Note, no default so compiler catches missing keys */
         case CUSTOM_KEY:
                 g_assert_not_reached ();
