@@ -43,7 +43,7 @@
 #define GSD_WACOM_STYLUS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_STYLUS, GsdWacomStylusPrivate))
 
 #define WACOM_TABLET_SCHEMA "org.gnome.settings-daemon.peripherals.wacom"
-#define WACOM_DEVICE_CONFIG_BASE "/org/gnome/settings-daemon/peripherals/wacom/%s/"
+#define WACOM_DEVICE_CONFIG_BASE "/org/gnome/settings-daemon/peripherals/wacom/%s-%s/"
 #define WACOM_STYLUS_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.stylus"
 #define WACOM_ERASER_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.eraser"
 #define WACOM_BUTTON_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.tablet-button"
@@ -245,7 +245,8 @@ gsd_wacom_tablet_button_new (const char               *name,
 			     const char               *id,
 			     const char               *settings_path,
 			     GsdWacomTabletButtonType  type,
-			     int                       group_id)
+			     int                       group_id,
+			     int                       idx)
 {
 	GsdWacomTabletButton *ret;
 
@@ -260,6 +261,7 @@ gsd_wacom_tablet_button_new (const char               *name,
 		g_free (button_settings_path);
 	}
 	ret->group_id = group_id;
+	ret->idx = idx;
 	ret->type = type;
 
 	return ret;
@@ -313,13 +315,17 @@ struct GsdWacomDevicePrivate
 
 	GsdWacomDeviceType type;
 	char *name;
-	char *icon_name;
+	char *path;
+	char *machine_id;
+	const char *icon_name;
 	char *tool_name;
 	gboolean reversible;
 	gboolean is_screen_tablet;
 	GList *styli;
 	GsdWacomStylus *last_stylus;
 	GList *buttons;
+	GHashTable *modes; /* key = int (group), value = int (index) */
+	GHashTable *num_modes; /* key = int (group), value = int (index) */
 	GSettings *wacom_settings;
 };
 
@@ -368,13 +374,15 @@ filter_events (XEvent         *xevent,
 	name = XGetAtomName (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), pev->property);
 	if (name == NULL ||
 	    g_strcmp0 (name, WACOM_SERIAL_IDS_PROP) != 0) {
+		if (name)
+			XFree (name);
 		return GDK_FILTER_CONTINUE;
 	}
 	XFree (name);
 
 	tool_id = xdevice_get_last_tool_id (device->priv->device_id);
 	if (tool_id == -1) {
-		g_warning ("Failed to get value for changed stylus ID on device '%d", device->priv->device_id);
+		g_warning ("Failed to get value for changed stylus ID on device '%d'", device->priv->device_id);
 		return GDK_FILTER_CONTINUE;
 	}
 	gsd_wacom_device_set_current_stylus (device, tool_id);
@@ -391,11 +399,11 @@ setup_property_notify (GsdWacomDevice *device)
 
 	evmask.deviceid = device->priv->device_id;
 	evmask.mask_len = XIMaskLen (XI_PropertyEvent);
-	evmask.mask = g_malloc0(evmask.mask_len * sizeof(char));
+	evmask.mask = g_new0 (guchar, evmask.mask_len);
 	XISetMask (evmask.mask, XI_PropertyEvent);
 
 	dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-	XISelectEvents (dpy, DefaultRootWindow(dpy), &evmask, 1);
+	XISelectEvents (dpy, DefaultRootWindow (dpy), &evmask, 1);
 
 	g_free (evmask.mask);
 
@@ -472,10 +480,10 @@ get_device_type (XDeviceInfo *dev)
                                  device, prop, 0, 1, False,
                                  XA_ATOM, &realtype, &realformat, &nitems,
                                  &bytes_after, &data);
-        if (gdk_error_trap_pop () || rc != Success || realtype == None) {
-                XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+
+        if (gdk_error_trap_pop () || rc != Success || realtype == None)
                 ret = WACOM_TYPE_INVALID;
-        }
 
         XFree (data);
 
@@ -485,7 +493,7 @@ get_device_type (XDeviceInfo *dev)
 /* Finds an output which matches the given EDID information. Any NULL
  * parameter will be interpreted to match any value.
  */
-static GnomeRROutputInfo*
+static GnomeRROutputInfo *
 find_output_by_edid (const gchar *vendor, const gchar *product, const gchar *serial)
 {
 	GError *error = NULL;
@@ -547,7 +555,7 @@ find_output_by_edid (const gchar *vendor, const gchar *product, const gchar *ser
 	return retval;
 }
 
-static GnomeRROutputInfo*
+static GnomeRROutputInfo *
 find_output_by_heuristic (GsdWacomDevice *device)
 {
 	GnomeRROutputInfo *rr_output_info;
@@ -556,37 +564,45 @@ find_output_by_heuristic (GsdWacomDevice *device)
 	 * tablets and may give the wrong result if multiple Wacom
 	 * display tablets are connected.
 	 */
-	rr_output_info = find_output_by_edid("WAC", NULL, NULL);
+	rr_output_info = find_output_by_edid ("WAC", NULL, NULL);
 	return rr_output_info;
 }
 
-static GnomeRROutputInfo*
+static GnomeRROutputInfo *
 find_output_by_display (GsdWacomDevice *device)
 {
 	gsize n;
 	GSettings *tablet;
 	GVariant *display;
 	const gchar **edid;
+	GnomeRROutputInfo *ret;
 
 	if (device == NULL)
 		return NULL;
 
+	ret      = NULL;
 	tablet   = device->priv->wacom_settings;
 	display  = g_settings_get_value (tablet, "display");
 	edid     = g_variant_get_strv (display, &n);
 
 	if (n != 3) {
 		g_critical ("Expected 'display' key to store %d values; got %"G_GSIZE_FORMAT".", 3, n);
-		return NULL;
+		goto out;
 	}
 
-	if (strlen(edid[0]) == 0 || strlen(edid[1]) == 0 || strlen(edid[2]) == 0)
-		return NULL;
+	if (strlen (edid[0]) == 0 || strlen (edid[1]) == 0 || strlen (edid[2]) == 0)
+		goto out;
 
-	return find_output_by_edid (edid[0], edid[1], edid[2]);
+	ret = find_output_by_edid (edid[0], edid[1], edid[2]);
+
+out:
+	g_free (edid);
+	g_variant_unref (display);
+
+	return ret;
 }
 
-static GnomeRROutputInfo*
+static GnomeRROutputInfo *
 find_output_by_monitor (GdkScreen *screen,
 			int        monitor)
 {
@@ -656,7 +672,7 @@ set_display_by_output (GsdWacomDevice    *device,
 	c_array = g_settings_get_value (tablet, "display");
 	g_variant_get_strv (c_array, &nvalues);
 	if (nvalues != 3) {
-		g_warning("Unable set set display property. Got %"G_GSIZE_FORMAT" items; expected %d items.\n", nvalues, 4);
+		g_warning ("Unable set set display property. Got %"G_GSIZE_FORMAT" items; expected %d items.\n", nvalues, 4);
 		return;
 	}
 
@@ -677,7 +693,7 @@ set_display_by_output (GsdWacomDevice    *device,
 	values[0] = o_vendor;
 	values[1] = o_product;
 	values[2] = o_serial;
-	n_array = g_variant_new_strv((const gchar * const *) &values, 3);
+	n_array = g_variant_new_strv ((const gchar * const *) &values, 3);
 	g_settings_set_value (tablet, "display", n_array);
 
 	g_free (o_vendor);
@@ -685,27 +701,25 @@ set_display_by_output (GsdWacomDevice    *device,
 	g_free (o_serial);
 }
 
-
 void
 gsd_wacom_device_set_display (GsdWacomDevice *device,
                               int             monitor)
 {
-	GnomeRROutputInfo *output;
+	GnomeRROutputInfo *output = NULL;
 
         g_return_if_fail (GSD_IS_WACOM_DEVICE (device));
-        g_return_if_fail (monitor >= 0);
 
-	output = find_output_by_monitor (gdk_screen_get_default (), monitor);
-	if (output != NULL)
-		set_display_by_output (device, output);
+	if (monitor >= 0)
+		output = find_output_by_monitor (gdk_screen_get_default (), monitor);
+	set_display_by_output (device, output);
 }
 
-static GnomeRROutputInfo*
+static GnomeRROutputInfo *
 find_output (GsdWacomDevice *device)
 {
 	GnomeRROutputInfo *rr_output_info;
 
-	rr_output_info = find_output_by_display(device);
+	rr_output_info = find_output_by_display (device);
 
 	if (rr_output_info == NULL) {
 		if (gsd_wacom_device_is_screen_tablet (device)) {
@@ -713,7 +727,7 @@ find_output (GsdWacomDevice *device)
 			if (rr_output_info == NULL) {
 				g_warning ("No fuzzy match based on heuristics was found.");
 			} else {
-				g_warning("Automatically mapping tablet to heuristically-found display.");
+				g_warning ("Automatically mapping tablet to heuristically-found display.");
 				set_display_by_output (device, rr_output_info);
 			}
 		}
@@ -759,7 +773,7 @@ gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 
         g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), -1);
 
-	rr_output_info = find_output(device);
+	rr_output_info = find_output (device);
 	if (rr_output_info == NULL)
 		return -1;
 
@@ -849,6 +863,30 @@ add_stylus_to_device (GsdWacomDevice *device,
 	}
 }
 
+int
+gsd_wacom_device_set_next_mode (GsdWacomDevice *device,
+				int             group_id)
+{
+	int current_idx;
+	int num_modes;
+
+	g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), -1);
+	current_idx = GPOINTER_TO_INT (g_hash_table_lookup (device->priv->modes, GINT_TO_POINTER(group_id)));
+	/* That means that the mode doesn't exist, see gsd_wacom_device_add_modes() */
+	g_return_val_if_fail (current_idx != 0, -1);
+
+	current_idx++;
+
+	num_modes = GPOINTER_TO_INT (g_hash_table_lookup (device->priv->num_modes, GINT_TO_POINTER(group_id)));
+
+	if (current_idx > num_modes)
+		current_idx = 1;
+
+	g_hash_table_insert (device->priv->modes, GINT_TO_POINTER (group_id), GINT_TO_POINTER (current_idx));
+
+	return current_idx;
+}
+
 static int
 flags_to_group (WacomButtonFlags flags)
 {
@@ -881,14 +919,18 @@ gsd_wacom_device_add_ring_modes (WacomDevice      *wacom_device,
 		for (i = 1; i <= num_modes; i++) {
 			name = g_strdup_printf (_("Left Ring Mode #%d"), i);
 			id = g_strdup_printf ("left-ring-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_RING_MODESWITCH)));
+			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_RING_MODESWITCH), i - 1));
+			g_free (name);
+			g_free (id);
 		}
 	} else if ((direction & WACOM_BUTTON_POSITION_RIGHT) && libwacom_has_ring2 (wacom_device)) {
 		num_modes = libwacom_get_ring2_num_modes (wacom_device);
 		for (i = 1; i <= num_modes; i++) {
 			name = g_strdup_printf (_("Right Ring Mode #%d"), i);
 			id = g_strdup_printf ("right-ring-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_RING2_MODESWITCH)));
+			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_RING2_MODESWITCH), i - 1));
+			g_free (name);
+			g_free (id);
 		}
 	}
 
@@ -916,14 +958,18 @@ gsd_wacom_device_add_strip_modes (WacomDevice      *wacom_device,
 		for (i = 1; i <= num_modes; i++) {
 			name = g_strdup_printf (_("Left Touchstrip Mode #%d"), i);
 			id = g_strdup_printf ("left-strip-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_TOUCHSTRIP_MODESWITCH)));
+			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_TOUCHSTRIP_MODESWITCH), i - 1));
+			g_free (name);
+			g_free (id);
 		}
 	} else if ((direction & WACOM_BUTTON_POSITION_RIGHT) && num_strips >= 2) {
 		num_modes = libwacom_get_strips_num_modes (wacom_device);
 		for (i = 1; i <= num_modes; i++) {
 			name = g_strdup_printf (_("Right Touchstrip Mode #%d"), i);
 			id = g_strdup_printf ("right-strip-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_TOUCHSTRIP2_MODESWITCH)));
+			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_TOUCHSTRIP2_MODESWITCH), i - 1));
+			g_free (name);
+			g_free (id);
 		}
 	}
 
@@ -977,7 +1023,7 @@ gsd_wacom_device_add_buttons_dir (WacomDevice      *wacom_device,
 
 		name = g_strdup_printf (button_str, button_num++);
 		id = g_strdup_printf ("%s%c", button_str_id, i);
-		l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_NORMAL, flags_to_group (flags)));
+		l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_NORMAL, flags_to_group (flags), -1));
 		g_free (name);
 		g_free (id);
 	}
@@ -985,27 +1031,27 @@ gsd_wacom_device_add_buttons_dir (WacomDevice      *wacom_device,
 	/* Handle modeswitches */
 	for (i = 'A'; i < 'A' + num_buttons; i++) {
 		WacomButtonFlags flags;
+		char *name, *id;
 
 		flags = libwacom_get_button_flag (wacom_device, i);
 		if (!(flags & direction))
 			continue;
 		/* Ignore non-mode switches */
-		if (flags & WACOM_BUTTON_MODESWITCH) {
-			char *name, *id;
+		if (!(flags & WACOM_BUTTON_MODESWITCH))
+			continue;
 
-			name = gsd_wacom_device_modeswitch_name (flags, button_num++);
-			id = g_strdup_printf ("%s%c", button_str_id, i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_HARDCODED, flags_to_group (flags)));
-			g_free (name);
-			g_free (id);
+		name = gsd_wacom_device_modeswitch_name (flags, button_num++);
+		id = g_strdup_printf ("%s%c", button_str_id, i);
+		l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_HARDCODED, flags_to_group (flags), -1));
+		g_free (name);
+		g_free (id);
 
-			if (flags & WACOM_BUTTON_RINGS_MODESWITCH)
-				l = g_list_concat (l, gsd_wacom_device_add_ring_modes (wacom_device, settings_path, direction));
-			else if (flags & WACOM_BUTTON_TOUCHSTRIPS_MODESWITCH)
-				l = g_list_concat (l, gsd_wacom_device_add_strip_modes (wacom_device, settings_path, direction));
-			else
-				g_warning ("Unhandled modeswitches");
-		}
+		if (flags & WACOM_BUTTON_RINGS_MODESWITCH)
+			l = g_list_concat (l, gsd_wacom_device_add_ring_modes (wacom_device, settings_path, direction));
+		else if (flags & WACOM_BUTTON_TOUCHSTRIPS_MODESWITCH)
+			l = g_list_concat (l, gsd_wacom_device_add_strip_modes (wacom_device, settings_path, direction));
+		else
+			g_warning ("Unhandled modeswitches");
 	}
 
 	return l;
@@ -1036,6 +1082,38 @@ gsd_wacom_device_add_buttons (GsdWacomDevice *device,
 	device->priv->buttons = ret;
 }
 
+static void
+gsd_wacom_device_add_modes (GsdWacomDevice *device,
+			    WacomDevice    *wacom_device)
+{
+	GList *l;
+
+	device->priv->modes = g_hash_table_new (g_direct_hash, g_direct_equal);
+	device->priv->num_modes = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+	for (l = device->priv->buttons; l != NULL; l = l->next) {
+		GsdWacomTabletButton *button = l->data;
+
+		if (button->group_id > 0)
+			g_hash_table_insert (device->priv->modes, GINT_TO_POINTER (button->group_id), GINT_TO_POINTER (1));
+
+		/* See flags_to_group() for group ID/button type matches */
+		if (button->group_id == 1) {
+			g_hash_table_insert (device->priv->num_modes,
+					     GINT_TO_POINTER (button->group_id),
+					     GINT_TO_POINTER (libwacom_get_ring_num_modes (wacom_device)));
+		} else if (button->group_id == 2) {
+			g_hash_table_insert (device->priv->num_modes,
+					     GINT_TO_POINTER (button->group_id),
+					     GINT_TO_POINTER (libwacom_get_ring2_num_modes (wacom_device)));
+		} else if (button->group_id == 3 || button->group_id == 4) {
+			g_hash_table_insert (device->priv->num_modes,
+					     GINT_TO_POINTER (button->group_id),
+					     GINT_TO_POINTER (libwacom_get_strips_num_modes (wacom_device)));
+		}
+	}
+}
+
 static void
 gsd_wacom_device_update_from_db (GsdWacomDevice *device,
 				 WacomDevice    *wacom_device,
@@ -1043,7 +1121,9 @@ gsd_wacom_device_update_from_db (GsdWacomDevice *device,
 {
 	char *settings_path;
 
-	settings_path = g_strdup_printf (WACOM_DEVICE_CONFIG_BASE, libwacom_get_match (wacom_device));
+	settings_path = g_strdup_printf (WACOM_DEVICE_CONFIG_BASE,
+					 device->priv->machine_id,
+					 libwacom_get_match (wacom_device));
 	device->priv->wacom_settings = g_settings_new_with_path (WACOM_TABLET_SCHEMA,
 								 settings_path);
 
@@ -1052,31 +1132,28 @@ gsd_wacom_device_update_from_db (GsdWacomDevice *device,
 	device->priv->is_screen_tablet = libwacom_is_builtin (wacom_device);
 	if (device->priv->is_screen_tablet) {
 		if (libwacom_get_class (wacom_device) == WCLASS_CINTIQ)
-			device->priv->icon_name = g_strdup ("wacom-tablet-cintiq");
+			device->priv->icon_name = "wacom-tablet-cintiq";
 		else
-			device->priv->icon_name = g_strdup ("wacom-tablet-pc");
+			device->priv->icon_name = "wacom-tablet-pc";
 	} else {
-		device->priv->icon_name = g_strdup ("wacom-tablet");
+		device->priv->icon_name = "wacom-tablet";
 	}
 
-	if (device->priv->type == WACOM_TYPE_PAD)
+	if (device->priv->type == WACOM_TYPE_PAD) {
 		gsd_wacom_device_add_buttons (device, wacom_device, settings_path);
+		gsd_wacom_device_add_modes (device, wacom_device);
+	}
 
 	if (device->priv->type == WACOM_TYPE_STYLUS ||
 	    device->priv->type == WACOM_TYPE_ERASER) {
-		int *ids;
+		const int *ids;
 		int num_styli;
 		guint i;
 
-		ids = libwacom_get_supported_styli(wacom_device, &num_styli);
+		ids = libwacom_get_supported_styli (wacom_device, &num_styli);
+		g_assert (num_styli >= 1);
 		for (i = 0; i < num_styli; i++)
 			add_stylus_to_device (device, settings_path, ids[i]);
-		/* Create a fallback stylus if we don't have one */
-		if (num_styli == 0)
-			add_stylus_to_device (device, settings_path,
-					      device->priv->type == WACOM_TYPE_STYLUS ?
-					      WACOM_STYLUS_FALLBACK_ID : WACOM_ERASER_FALLBACK_ID);
-
 		device->priv->styli = g_list_reverse (device->priv->styli);
 	}
 	g_free (settings_path);
@@ -1093,7 +1170,6 @@ gsd_wacom_device_constructor (GType                     type,
         WacomDevice *wacom_device;
         int n_devices;
         guint i;
-        char *path;
 
         device = GSD_WACOM_DEVICE (G_OBJECT_CLASS (gsd_wacom_device_parent_class)->constructor (type,
 												n_construct_properties,
@@ -1126,8 +1202,8 @@ gsd_wacom_device_constructor (GType                     type,
 	if (device->priv->type == WACOM_TYPE_INVALID)
 		goto end;
 
-	path = xdevice_get_device_node (device->priv->device_id);
-	if (path == NULL) {
+	device->priv->path = xdevice_get_device_node (device->priv->device_id);
+	if (device->priv->path == NULL) {
 		g_warning ("Could not get the device node path for ID '%d'", device->priv->device_id);
 		device->priv->type = WACOM_TYPE_INVALID;
 		goto end;
@@ -1136,31 +1212,29 @@ gsd_wacom_device_constructor (GType                     type,
 	if (db == NULL)
 		db = libwacom_database_new ();
 
-	wacom_device = libwacom_new_from_path (db, path, FALSE, NULL);
+	wacom_device = libwacom_new_from_path (db, device->priv->path, FALSE, NULL);
 	if (!wacom_device) {
 		WacomError *wacom_error;
 
 		g_debug ("Creating fallback driver for wacom tablet '%s' ('%s')",
 			 gdk_device_get_name (device->priv->gdk_device),
-			 path);
+			 device->priv->path);
 
 		wacom_error = libwacom_error_new ();
-		wacom_device = libwacom_new_from_path (db, path, TRUE, wacom_error);
+		wacom_device = libwacom_new_from_path (db, device->priv->path, TRUE, wacom_error);
 		if (wacom_device == NULL) {
 			g_warning ("Failed to create fallback wacom device for '%s': %s (%d)",
-				   path,
+				   device->priv->path,
 				   libwacom_error_get_message (wacom_error),
 				   libwacom_error_get_code (wacom_error));
-			g_free (path);
 			libwacom_error_free (&wacom_error);
 			device->priv->type = WACOM_TYPE_INVALID;
 			goto end;
 		}
 	}
 
-	gsd_wacom_device_update_from_db (device, wacom_device, path);
+	gsd_wacom_device_update_from_db (device, wacom_device, device->priv->path);
 	libwacom_destroy (wacom_device);
-	g_free (path);
 
 	if (device->priv->type == WACOM_TYPE_STYLUS ||
 	    device->priv->type == WACOM_TYPE_ERASER) {
@@ -1242,6 +1316,12 @@ gsd_wacom_device_init (GsdWacomDevice *device)
 {
         device->priv = GSD_WACOM_DEVICE_GET_PRIVATE (device);
         device->priv->type = WACOM_TYPE_INVALID;
+
+        if (g_file_get_contents ("/etc/machine-id", &device->priv->machine_id, NULL, NULL) == FALSE)
+                if (g_file_get_contents ("/var/lib/dbus/machine-id", &device->priv->machine_id, NULL, NULL) == FALSE)
+                        device->priv->machine_id = g_strdup ("00000000000000000000000000000000");
+
+        device->priv->machine_id = g_strstrip (device->priv->machine_id);
 }
 
 static void
@@ -1264,6 +1344,9 @@ gsd_wacom_device_finalize (GObject *object)
                 p->wacom_settings = NULL;
         }
 
+        g_list_foreach (p->styli, (GFunc) g_object_unref, NULL);
+        g_list_free (p->styli);
+
         g_list_foreach (p->buttons, (GFunc) gsd_wacom_tablet_button_free, NULL);
         g_list_free (p->buttons);
 
@@ -1273,8 +1356,20 @@ gsd_wacom_device_finalize (GObject *object)
         g_free (p->tool_name);
         p->tool_name = NULL;
 
-        g_free (p->icon_name);
-        p->icon_name = NULL;
+        g_free (p->path);
+        p->path = NULL;
+
+        g_free (p->machine_id);
+        p->machine_id = NULL;
+
+        if (p->modes) {
+                g_hash_table_destroy (p->modes);
+                p->modes = NULL;
+        }
+        if (p->num_modes) {
+                g_hash_table_destroy (p->num_modes);
+                p->num_modes = NULL;
+        }
 
 	gdk_window_remove_filter (NULL,
 				  (GdkFilterFunc) filter_events,
@@ -1324,6 +1419,14 @@ gsd_wacom_device_get_name (GsdWacomDevice *device)
 	return device->priv->name;
 }
 
+const char *
+gsd_wacom_device_get_path (GsdWacomDevice *device)
+{
+	g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), NULL);
+
+	return device->priv->path;
+}
+
 const char *
 gsd_wacom_device_get_icon_name (GsdWacomDevice *device)
 {
@@ -1461,7 +1564,7 @@ gsd_wacom_device_get_area (GsdWacomDevice *device)
 
 	device_area = g_new0 (int, nitems);
 	for (i = 0; i < nitems; i++)
-		device_area[i] = ((long*)data)[i];
+		device_area[i] = ((long *)data)[i];
 
 	XFree (data);
 	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
@@ -1513,11 +1616,28 @@ find_button_with_id (GsdWacomDevice *device,
 	return NULL;
 }
 
+static GsdWacomTabletButton *
+find_button_with_index (GsdWacomDevice *device,
+			const char     *id,
+			int             index)
+{
+	GsdWacomTabletButton *button;
+	char *str;
+
+	str = g_strdup_printf ("%s-mode-%d", id, index);
+	button = find_button_with_id (device, str);
+	g_free (str);
+
+	return button;
+}
+
 GsdWacomTabletButton *
 gsd_wacom_device_get_button (GsdWacomDevice   *device,
 			     int               button,
 			     GtkDirectionType *dir)
 {
+	int index;
+
 	if (button <= 26) {
 		char *id;
 		GsdWacomTabletButton *ret;
@@ -1553,20 +1673,24 @@ gsd_wacom_device_get_button (GsdWacomDevice   *device,
 		;;
 	}
 
-	/* FIXME handle the mode */
+	/* The group ID is implied by the button number */
 	switch (button) {
 	case 90:
 	case 91:
-		return find_button_with_id (device, "left-ring-mode-1");
+		index = GPOINTER_TO_INT (g_hash_table_lookup (device->priv->modes, GINT_TO_POINTER (1)));
+		return find_button_with_index (device, "left-ring", index);
 	case 92:
 	case 93:
-		return find_button_with_id (device, "right-ring-mode-1");
+		index = GPOINTER_TO_INT (g_hash_table_lookup (device->priv->modes, GINT_TO_POINTER (2)));
+		return find_button_with_index (device, "right-ring", index);
 	case 94:
 	case 95:
-		return find_button_with_id (device, "left-strip-mode-1");
+		index = GPOINTER_TO_INT (g_hash_table_lookup (device->priv->modes, GINT_TO_POINTER (3)));
+		return find_button_with_index (device, "left-strip", index);
 	case 96:
 	case 97:
-		return find_button_with_id (device, "right-strip-mode-1");
+		index = GPOINTER_TO_INT (g_hash_table_lookup (device->priv->modes, GINT_TO_POINTER (4)));
+		return find_button_with_index (device, "right-strip", index);
 	default:
 		return NULL;
 	}
